/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

#include "fingerprint.h"

static int choose_next_centre ( int & icentre,int * selected,int size)
{
  for(int i=0;i<size;++i)
  {
    if(! selected[i])
    {
      icentre = i;
      return 1;
    }
  }
  return 0;
}

static void usage(int rc)
{
  cerr << "Performs leader clustering on a set of fingerprints\n";
  cerr << "Usage <options> <input_file>\n";
  cerr << "options:\n";
  cerr << " -t <dis>       specify distance threshold (default 0.2)\n";
  cerr << " -A <file>      specify previously selected fingerprints\n";

  exit (rc);
}


int
main (int argc, char ** argv)
{

  float distance_threshold = 0.2F;
  char previously_selected_file[200];
  int previous = 0;
  int i;
  for( i=1;(i<argc) && ((argv[i])[0] == '-');i++)
  {
    switch ((argv[i])[1])
    {
      case 'h': usage(0);
      case 't': i++; distance_threshold = atof(argv[i]); break;
      case 'A': i++; previous=1; strcpy(previously_selected_file,argv[i]); break;
      default: 
		cerr<<"\nUnrecognized option "<< argv[i] <<endl<<endl;
		usage(0);
    }
  }
  if(i != (argc-1) ) 
  {
    cerr << "No input fp file !"<<endl;
    usage(0);
  }

  Fingerprint * pool = NULL;

  int size=build_fp_pool(argv[i],pool);
  cerr <<"Pool has "<< size << " fingerprints." <<endl;
  int *selected=new int[size];
  for( i=0;i<size;i++)
    selected[i] = 0;
  
  int cluster_id = 0;
  if(previous)
  {
     Fingerprint * pool2 = NULL;
     int size2 = build_fp_pool(previously_selected_file,pool2);
     cerr <<"Previously selected Pool has "<< size2 << " fingerprints." <<endl;
     for(int j =0; j<size2;j++)
     {
       for( i=0;i<size;i++)
       {
	 if(selected[i])
	   continue;
	 float dis= tanimoto_distance(pool2[j],pool[i]);
	 if(dis > distance_threshold)
	   continue;
	 selected[i] = 1;
	 cout<<cluster_id <<" "<<pool[i].id()<<" "<<dis<<endl;
       }
     }
     delete [] pool2;
  }
 
  int icentre=0;
  int start =0 ;
  while (choose_next_centre (icentre,&selected[start],size - start))
  {
    cluster_id ++;
    icentre += start;
    cout<<cluster_id <<" "<<pool[icentre].id()<<" "<<0.0F<<endl;
    start = icentre + 1;
    for( i= start;i<size;++i)
    {
      if(selected[i])
        continue;
      float dis= tanimoto_distance(pool[icentre],pool[i]);
      if(dis > distance_threshold)
        continue;
      selected[i] = 1;
      cout<<cluster_id <<" "<<pool[i].id()<<" "<<dis<<endl;
    }
  }
  cerr << "Find " << cluster_id << " clusters"<<endl;
  delete [] selected;
  delete [] pool;
  
  return 0;
}
