/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

#include "fingerprint.h"


static int furthest_from_already_selected(int & ichoose, const int *nsn, const float * dis,int size)
{
  ichoose = -1;
  float max_dis = -0.5F;
  for(int i=0;i<size;++i)
  {
    if( dis[i] > max_dis)
    {
      max_dis = dis[i];
      ichoose = i;
    }
  }

  return (ichoose  >= 0 );
}

static void usage(int rc)
{
  cerr << "Performs spread selection on a set of fingerprints\n";
  cerr << "Usage <options> <input_file>\n";
  cerr << "options:\n";
  cerr << " -n <number>    specify number of fingerprints to be selected\n";
  cerr << " -A <file>      specify previously selected fingerprints\n";

  exit (rc);
}

  
int
main (int argc, char ** argv)
{
 
  int  number  = 0;
  char previously_selected_file[200];
  int previous = 0;
  int i;
  for( i=1;(i<argc) && ((argv[i])[0] == '-');i++)
  {
    switch ((argv[i])[1])
    {
      case 'h': usage(0);
      case 'n': i++; number = atoi(argv[i]); break;
      case 'A': i++; previous=1; strcpy(previously_selected_file,argv[i]); break;
      default: 
		cerr<<"\nUnrecognized option "<< argv[i] <<endl<<endl;
		usage(0);
    }
  }
  if(i != (argc-1) ) 
  {
    cerr << "No input fp file !"<<endl;
    usage(0);
  }

  Fingerprint * pool = NULL;

  int size=build_fp_pool(argv[i],pool);
  cerr <<"Pool has "<< size << " fingerprints." <<endl;
  
  int * nsn = new int[size]; 
  float * min_dis = new float[size];
  for(i=0;i<size;i++)
  {
    nsn [i] =0;
    min_dis[i] = 2.0F;
  }
  int ichoose =0;
  int selected =1 ;
  Fingerprint * pool2 = NULL;
  if(previous)
  {
     int size2 = build_fp_pool(previously_selected_file,pool2);
     cerr <<"Previously selected Pool has "<< size2 << " fingerprints." <<endl;
     for(int j =0; j<size2;j++)
     {
       for(i=0;i<size;i++)
       {
         float dis= tanimoto_distance(pool2[j],pool[i]);
	 if(dis < min_dis[i] )
	 {
	   min_dis[i] = dis;
	   nsn[i] = -(j+1);
	 }
       }
     }
     furthest_from_already_selected(ichoose,nsn,min_dis,size);
     cout <<selected<<" "<< pool[ichoose].id()<<" "<<pool2[-nsn[ichoose]-1].id()<<" "<<min_dis[ichoose]<<endl;
  }
  else
  {
    cout <<selected<<" "<< pool[ichoose].id()<<" "<<"*"<<" "<<"*"<<endl;
  }
  if(number <= 0) number = size;
  while(selected < number)
  {
     min_dis[ichoose] = -1.0F;
     for(i = 0; i < size; ++i)
     {
       if(min_dis[i] < -0.5F)
	 continue;
       float dis= tanimoto_distance(pool[ichoose],pool[i]);
       if(dis < min_dis[i] )
       {
	 min_dis[i] = dis;
	 nsn[i] = ichoose;
       }

     }
     if(!furthest_from_already_selected(ichoose,nsn,min_dis,size))
     {
        break;
     }
     ++selected;
     cout <<selected<<" "<< pool[ichoose].id()<<" "<< (nsn[ichoose]<0?pool2[-nsn[ichoose]-1].id():pool[nsn[ichoose]].id())<<" "<<min_dis[ichoose]<<endl;
  }
  cerr << "Selected " << selected << " fingerprints"<<endl;

  if(pool2)
    delete [] pool2;
  delete [] nsn;
  delete [] min_dis;
  delete [] pool;
  
  return 0;
}




