/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/


#include "fingerprint.h"
#include "precompbit8.h"

#include <fstream>

#define HTOI(a) ((a >= 'A' )?(a - 'A' + 10 ):(a - '0'))


void Fingerprint::clear()
{
  if(_words>0)
  {
    delete []_needle;
    _words =0;
  }
}

Fingerprint::Fingerprint() : _needle(0),_words(0)
{
}

Fingerprint::~Fingerprint()
{
  clear();
}

int Fingerprint::init(string id, string hex)
{
  clear();
  _id = id;
  int nfp = count(hex.begin(),hex.end(),',') + 1;

  _words=1+ hex.size()/8 + nfp * 3 ; // slightly large than what we needed...

  _needle = new unsigned int [_words];
  _needle[0] = nfp;

  int pos = 1;
  istringstream iss(hex);
  string pch;
  while(getline(iss,pch,','))
  {
    int n = pch.size();
    int m = n - n%8 + (n%8?8:0);
    _needle[pos] = m/8; pos++;
    unsigned char * byte = (unsigned char * ) (&(_needle[pos]));
    int bits=0;
    for(int i=0;i<m;)
    {
       char a = i<n?pch[i]:'0'; i++;
       char b = i<n?pch[i]:'0'; i++;
       (*byte) = HTOI(a)*16+HTOI(b);
       bits += eight_bit_count[*byte];
       byte++;
    }
    pos += m/8 ;
    _needle[pos] = bits; pos ++;
  }
  _words=pos;
  return nfp;
}

int build_fp_pool (const char * fname, Fingerprint * & pool)
{
  ifstream fin(fname);
  int lines = count(istreambuf_iterator<char>(fin),istreambuf_iterator<char>(),'\n');
  if(lines ==0)
  {
    cerr << "There's no fingerprints in '" <<fname <<"'"<<endl;
    exit(1);
  }
  cerr << fname <<" has " << lines <<" lines." <<endl;
  pool=0;
  pool = new Fingerprint[lines];
  if(pool==0)
  {
    cerr << "Out of CPU memory"<<endl;
    exit(1);
  }
  fin.seekg(0);
  int n =0;
  string id;
  string hex;
  while(getline(fin,id,' ') && getline(fin,hex))
  { 
    if(!pool[n].init(id,hex))
    {
      cerr<< "Fail to initilize " << id << " with fingerprint " << hex <<endl;
    }
    else if(pool[n].words() != pool[0].words())
    {
      cerr << "Fingerprint " <<id<<" has different length with the first one" <<endl;
      exit(1);
    }
    else
    {
      n++;
    }
  }
  fin.close();
  return n;
}

float tanimoto_distance(Fingerprint & fpa, Fingerprint & fpb)
{
  int m,pos=1,common,sum;
  unsigned int * pa = fpa.get_ptr();
  unsigned int * pb = fpb.get_ptr();
  float rc =0.0F;
  int n = pa[0];
  for(int i=0;i<n;++i)
  {
     common =0;
     m=pa[pos] *4 ; ++pos ;
     unsigned char * byte_a = (unsigned char * ) (&(pa[pos]));
     unsigned char * byte_b = (unsigned char * ) (&(pb[pos]));
     for(int j =0; j<m;++j)
     {
       common += eight_bit_count[(*byte_a) & (*byte_b)];
       ++byte_a;
       ++byte_b;
     }
     
     pos += m/4 ;
     sum = pa[pos] + pb[pos] -common;
     if(sum > 0)
       rc += (float)common/((float)sum);
     else
       rc += 1.0F;
     ++pos;
  }
  return 1.0F - rc/((float)n);
}

