/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

#ifndef _FINGERPRINT_H
#define _FINGERPRINT_H

#include <sstream>
#include <string>
#include <iostream>
#include <cstdlib>
#include <algorithm>
#include <string.h>
using namespace std;

////////////////////////////////////////////////////
//object for single or multiple fix-length fingerprints
//the data is stored in a vector of integers ( _needle) as following:
//
//   NA*********BA*******B
//
//   where N is the number of fingerprints
//   A*******B is for each single fingerprint
//     where A is the number of integers 
//     **** is the 01 bits that stored in integers
//     B is the precomputed sum of 1-bit
////////////////////////////////////////////////////////////

class Fingerprint
{
  protected:
    string _id; //unique identifier
    unsigned int *_needle;  //continuous storage of all data
    int _words; //length of _needle

    void clear();

  private: //disable copy 
    Fingerprint(const Fingerprint &fp);
    Fingerprint& operator=(const Fingerprint &fp);

  public:
    Fingerprint();
    ~Fingerprint();

    int init(string id, string hex);

    int valid(){return _words >0 && _needle[0] > 0;}

    const char * id() {return _id.c_str();}
    
    int words() {return _words;}

    unsigned int * get_ptr() {return _needle;}
};

int build_fp_pool (const char * fname, Fingerprint * & pool);

float tanimoto_distance(Fingerprint & fpa, Fingerprint & fpb);

#endif

