/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

#ifndef _GPU_MISC_H
#define _GPU_MISC_H

#include <iostream>
using namespace std;

//warpper of gpu related functions, 
//

//memory
void * malloc_gpu(unsigned int size);
void * malloc_cpu(unsigned int size); //CPU pinned memory
void free_gpu(void *p);
void free_cpu(void *p);

//copy data 
void cpu_to_gpu(void *gpu, void *cpu,unsigned int size,int async);
void gpu_to_cpu(void *cpu, void *gpu,unsigned int size,int async);
void gpu_to_gpu(void *dst, void *src,unsigned int size,int async);

void sync_gpu();

void
gpu_tanimoto_distance_row(float *g_results,unsigned int *g_haystack,
                        int cols,int pool_size,
                        unsigned int *g_needle,int words,int column_order);

void gpu_sequence_int(int *v,int n,int start=0,int step=1);

void gpu_fill_float(float *v, int n, float c);

void cpu_sort_by_key(float *dis,int * idx, int n);
void gpu_sort_by_key(float *dis,int * idx, int n);

//find min of vector
int gpu_min_int(int * v,int n);


//for leader ...

void
gpu_tanimoto_distance_row_leader_threshold(float *dis_from_leader, float threshold,
                        int* cluster_number,int new_id,
                        unsigned int *g_haystack,
                        int cols,int pool_size,
                        unsigned int *g_needle,int words,int column_order);

int 
gpu_squeeze_pool_leader(unsigned int * data, int * index, 
     int * cluster, float * dis, unsigned int * scan,
      int * index2,float * dis2, 
     int n,int words,int ceil,int shifted);

void gpu_leader_final_sort(int * index,int * cluster, float * dis,int * index2, int n);

//for spread ...

void
gpu_tanimoto_distance_row_spread(float *min_dis,int * nsn, int nsn_id,
                        unsigned int *g_haystack,
                        int cols,int pool_size,
                        unsigned int *g_needle,int words,int column_order);

//find the index of max value
int gpu_max_index_float(float *dis,int n);

int 
gpu_squeeze_pool_spread(unsigned int * data,int * index, 
     int * nsn, float * dis, unsigned int * scan,
     int * index2,float * dis2, 
     int n,int words,int ceil,int shifted);

#endif

