/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/


#include "gpu_haystack.h"

void GPU_Haystack::clear()
{
  if(size()>0)
  {
    free_gpu(_d_data);
    free_gpu(_d_dis);
    free_gpu(_d_index);
    free_gpu(_d_scan);
    free_gpu(_d_dis_tmp);
    free_gpu(_d_index_tmp);
    _mols=0;
    _ceil=0;
    _words=0;
  }
}

GPU_Haystack::GPU_Haystack():_mols(0),_ceil(0),_words(0),_start(0),_end(0),_shifted(0) 
{
}

GPU_Haystack::~GPU_Haystack()
{
  clear();
}

int GPU_Haystack::init(Fingerprint * pool, int pool_size)
{
  clear();
  _mols=pool_size;
  _words=pool[0].words();
  _ceil= ((_mols%32)?(_mols+32-(_mols%32)):_mols);
  _d_data=(unsigned int*)malloc_gpu(size()*sizeof(unsigned int));
  _d_dis=(float *)malloc_gpu(_mols * sizeof(float));
  _d_index=(int *)malloc_gpu(_mols * sizeof(int));
  gpu_sequence_int(_d_index,_mols,0,1);
  _d_scan=(unsigned int *)malloc_gpu((_mols +1) * sizeof(unsigned int)); //one more items for inclusive scan
  _d_dis_tmp=(float *)malloc_gpu(_mols * sizeof(float));
  _d_index_tmp=(int *)malloc_gpu(_mols * sizeof(int));
  unsigned int *data= new unsigned int[size()];
  for(int i=0;i<pool_size;i++)
  {
    unsigned int *p = pool[i].get_ptr();
    for(int j=0; j<_words;j++)
    {
      data[j*_ceil+i] = p[j];
    }
  }
  cpu_to_gpu(_d_data,data,size()*sizeof(unsigned int),0);
  unsigned int tmp=0;
  cpu_to_gpu(_d_scan,&tmp,sizeof(unsigned int),0);
  delete []data;
  _shifted=0;
  _start=0;
  _end=_mols;

  return 1;
}

