/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

#ifndef _GPU_HAYSTACK_H
#define _GPU_HAYSTACK_H

#include "gpu_needle.h"


//object that store a pool of Fingerprint on GPU
//the Fingerprints are stored in _d_data on GPU, in column-first order
//
//for consistency, we use distance everywhere...
//distance = 1 - tanimoto_similarity
//
//variable start with _d_ are on GPU, others are on CPU
//

class GPU_Haystack
{
  protected:
    int _mols; //number of molecules 
    int _ceil; //width of _d_data matrix, align to 32X
    int _words;//length of each Fingerprint data 

    /* allow calc part of the pool, form _start to _end */
    int _start;
    int _end;

    unsigned int * _d_data; //data matrix on GPU
    float * _d_dis; // store one row of distances on GPU
    int * _d_index; // store the original index of each molecule

    unsigned int * _d_scan;//temp vector for GPU algorithms
    float *_d_dis_tmp; // temp distance vector for GPU algorithms
    int *_d_index_tmp; // temp index vector for GPU algorithms
 
    int _shifted;//_d_data has one more rows than _words, we use _shifted to label if the _d_data is started with row 0 or started with row 1 (when we squeeze the pool, we shift the _d_data row by row)

    void clear();
    int size(){return _ceil*(_words+1);}  // the size of _d_data 
    
    void set_start(int start){_start=start;}
    void set_end(int end){_end=end;}
    int start(){return _start;}
    int end(){return _end;}
    int range(){return _end-_start;}

  private: //disable copy
    GPU_Haystack(const GPU_Haystack &);
    GPU_Haystack& operator=(const GPU_Haystack &);
  
  public:
    GPU_Haystack();
    virtual ~GPU_Haystack();
    
    int mols(){return _mols;}
    int words(){return _words;}
   
    int init(Fingerprint * pool, int pool_size);

};


#endif

