/**************************************************************************

    Copyright (C) 2010  Eli Lilly and Company

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

#ifndef _GPU_HAYSTACK_LEADER_H
#define _GPU_HAYSTACK_LEADER_H

#include "gpu_haystack.h"

//GPU_Haystack for Leader algorithm

class GPU_Haystack_Leader : public GPU_Haystack 
{
  protected:
    int * _d_cluster_number;//the assigned cluster number of each molecule (GPU)
    int * _cluster_number; //CPU
    
    float * _d_dis_from_leader;//distance from leader (GPU)
    float * _dis_from_leader;//CPU

    int * _index; //index on CPU ( versus GPU_Haystack::_d_index ) 
 
    int _cluster_id;//the current cluster number 

    void clear();

  private: //disable copy
    GPU_Haystack_Leader( const GPU_Haystack_Leader &);
    GPU_Haystack_Leader& operator=(const GPU_Haystack_Leader &);

  public:
    GPU_Haystack_Leader();
    ~GPU_Haystack_Leader();
    
    int init(Fingerprint* pool, int pool_size);
    
    void do_previously_selected(GPU_Needle & needle,float threshold); 
 
    int choose_next_centre(int & icentre);
    void do_leader(int icentre,float threshold);
   
    int final_process();

    int squeeze();//squeeze the pool

    int cluster_id(int i){return _cluster_number[i];}
    int pool_index(int i){ return _index[i];}
    float dis_from_leader(int i){return _dis_from_leader[i];}

    int leader(Fingerprint * pool,float  threshold); //main loop
};



#endif
